package optimierer;


/**
 * Testimplementierung des modifizierten Gradientenverfahrens.
 */
public class ModGradTest
{
    /**
     * param0: Ausgangspunkt (Anfangswerte der Parameter)
     * eps: Untere Grenze für delta (Abbruchkriterium)
     * delta0: Änderung der Parameter bei der Bildung von Variationen zu Beginn
     * faktor: zur Änderung von delta
     */
    public double[] optimieren(double[] param0, double eps, double delta0, double faktor)
    {
        double aktueller_fehler = berechneFehler(param0);
        
        double bester_neuer_fehler = aktueller_fehler;
        double aktuelles_delta =  delta0;
        
        double[] param_best = new double[param0.length];
        for(int i=0;i<param0.length;i++)
             param_best[i] = param0[i];
        
        double[] param_aktuell = new double[param0.length];
        double[] param_neu = new double[param0.length];
        double fehler_aktuell;
        double fehler_neu;
        
        int schritte = 0;
        
        while(aktuelles_delta>=eps)             
        {
            System.out.println("Schritt Nr. "+schritte+"Bislang bester Fehler:"+bester_neuer_fehler);
            for(int i=0;i<param0.length;i++)
                 System.out.println("p"+i+":"+param_best[i]);
            
            
            for(int i=0;i<param0.length;i++)
                 param_aktuell[i] = param_best[i];
            param_aktuell[0]+=aktuelles_delta;
            fehler_aktuell = berechneFehler(param_aktuell);
            
            for(int i=0;i<param0.length*2;i++)
            {
                for(int k=0;k<param0.length;k++)
                   param_neu[k] = param_best[k];
                if(i%2==0) //delta addieren
                    param_neu[i/2]+=aktuelles_delta;                      
                else //delta subtrahieren
                    param_neu[i/2]-=aktuelles_delta;                      
                fehler_neu = berechneFehler(param_neu);
                
                if(fehler_neu<fehler_aktuell)
                {
                    fehler_aktuell = fehler_neu;
                    
                    for(int k=0;k<param0.length;k++)
                       param_aktuell[k] = param_neu[k];
                    
                }
            }
            //An dieser Stelle habe ich den kleinsten Fehler bei allen Variationen
            //und muss jetzt vergleichen, ob dieser kleiner ist, als der des
            //Ausgangspunktes, also der des bislang besten Parametersatzes
            if(fehler_aktuell<=bester_neuer_fehler)
            {
                bester_neuer_fehler=fehler_aktuell;
                for(int k=0;k<param0.length;k++)
                   param_best[k] = param_aktuell[k];
                aktuelles_delta*=faktor;                
            }
            else //falls keine Variante besser ist, als der aktuelle Ausgangspunkt
            {
                aktuelles_delta/=faktor;  //delta wird verkleinert, neuer Ausgangspunkt ist alter.
            }
            
            schritte++;
        }
        
        return param_best;
    }

    /**
     * Hier würde normalerweise ein komplette Simulation mit dem aktuellen
     * Parametersatz durchgeführt werden.
     */
    public double berechneFehler(double[] param)
    {
        double a = param[0];
        double b = param[1];
        return a*a*a - 4.0*a + b*b*b - 16.0*b;
    }
    
    public static void main(String[] args)
    {
        ModGradTest mgt = new ModGradTest();
        double[] param0 = {10.0,10.0};
        double eps = 0.00001;
        double delta0 = 0.1;
        double faktor = 1.1;
        double[] best_param = mgt.optimieren(param0,eps,delta0,faktor);
        
        System.out.println("Bester Parametersatz:");
        for(int i=0;i<best_param.length;i++)
            System.out.println("Parameter Nr. "+i+"="+best_param[i]);
    }
}
